@@ -11,7 +11,7 @@ If PDL is about arrays, PDL::Dims turns them into hashes
 
 =cut
 
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 
 use strict;
 
@@ -341,7 +341,7 @@ sub initdim {
 				.dimsize($self,$d))
 				unless (approx(pdl ($p{max}-$p{min}) , pdl((dimsize($self,$d)-1)*$p{inc} )));
 		} elsif ($p{inc}) {
-			$p{max}=$p{inc}*dimsize($self,$d)+$p{min};
+			$p{max}=$p{inc}*(dimsize($self,$d)-1)+$p{min};
 		} elsif ($p{max}) {
 			$p{inc}=($p{max}-$p{min})/((dimsize($self,$d)-1)||1);
 		} else {
@@ -521,7 +521,10 @@ sub pos2i{
 	my $i=shift ; # value
 	barf "Unknown dim $d" unless (ref $self->hdr->{$d} eq  'HASH');
 	if (spacing($self,$d)) {
-		return rint (($i-dmin($self,$d))/dinc($self,$d)); 
+		my $res=rint (($i-dmin($self,$d))/dinc($self,$d)); 
+		$res=dimsize($self,$d)-1 if ($res>=dimsize($self,$d));
+		$res=0 if ($res<0);
+		return $res;
 	} else {
 		#say "searching for $i ",$self->hdr->{$d}->{num},".";
 		#say "Num? ",dnumeric($self,$d,undef);
@@ -659,14 +662,18 @@ sub nop { # wrapper to functions like sin, exp, rotate operating on one named di
 	my $op=shift;
 	my $dim=shift;
 	my @a=@_;
-	my @n=@{dimname($self)};
+	my $res=pdl $self;
+	if ($self->isa('PDL')) {
+		my @n=@{dimname($self)};
 	#say "nop: (self) @n";
 	#say "nop: ",diminfo $self;
 	#my $arg=shift; # suitably shaped second argument 
 	#say $self;
 	#say "dim $dim, pos ",didx($self,$dim)," ",%{$self->hdr->{$dim}};
-	$dim=dimname($self,0) unless defined $dim; # trivial 
-	my $res=$self->mv(didx($self,$dim),0);
+
+		$dim=dimname($self,0) unless defined $dim; # trivial 
+		$res=$res->mv(didx($self,$dim),0);
+	}
 	if ($op eq 'rotate'){
 		my $s=shift;
 		#say "schifing $res by $s";
@@ -680,8 +687,10 @@ sub nop { # wrapper to functions like sin, exp, rotate operating on one named di
 			$res=&$op($res,@a);
 		}
 	}
-	$res=$res->mv(0,didx($self,$dim));
-	$res->sethdr($self->hdr_copy);
+	if ($self->isa('PDL')) {
+		$res=$res->mv(0,didx($self,$dim));
+		$res->sethdr($self->hdr_copy);
+	}
 	#say "self $self, op $res, mv ",$res->mv(0,didx($res,$dim));
 	#say "nop: return", $self->info,@{dimname($res)},$res->info;
 	return ($res);
@@ -700,10 +709,12 @@ sub ncop { # named combine operation -- +*-/ ...
 		} else {
 			$res=&$op($self,$other,@_);
 		}		
-		$res->sethdr($other->hdr_copy) if (eval {$other->isa('PDL')});# both are scalar
+		if (eval {$other->isa('PDL')}) {# both are scalars ?	
+			$res->sethdr($other->hdr_copy); 
 		#say "ncop: other ",diminfo $other;
-		barf "ncop: $op changed dimensions. Use nagg or nreduce for aggregate functions"
-			unless ($other->nelem==$res->nelem);
+			barf "ncop: $op changed dimensions. Use nagg or nreduce for aggregate functions"
+				unless ($other->nelem==$res->nelem);
+		}
 		#say "ncop: res ",diminfo $res;
 		return $res;
 	}
@@ -1159,9 +1170,10 @@ removes a dminenso
 
 copies the diminfo from one piddle to the next.
 
-	copy_dim($a,$b,$dim,[$pos]);
+	copy_dim($a,$b,[$dim,[$pos]]);
 
-calls initdim on $b with parameters from $a. You can supply a new position.
+calls initdim on $b with parameters from $a. You can supply a new position. If you 
+call it with just two arguments, all dims are copied.
 
 =head2 sln
 
